;**************************************************************
; The LCD Module Subroutines                                      
;**************************************************************

;**************************************************************
;*SendChar - Sends character to LCD                                
;*This routine splits the character into the upper and lower        
;*nibbles and sends them to the LCD, upper nibble first.           
;**************************************************************
;
sch_b
Send_Char
	movwf   Char            ; Character to be sent is in W
	call    Wait_Busy       ; Wait for LCD to be ready
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	movf    Char,w          
	andlw   0xF0            ; Get upper nibble
	iorwf   LCD_DATA,F      ; Send data to LCD
	bcf     RW   		; Set LCD to write
	bsf     RS    		; Set LCD to data mode
	bsf     E     		; toggle E for LCD
	bcf     E
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	swapf   Char,W
	andlw   0xF0            ; Get lower nibble
	iorwf   LCD_DATA,F      ; Send data to LCD
	bsf     E     		; toggle E for LCD
	bcf     E
	return

;**************************************************************
;* Send_Cmd - Sends command to LCD                              
;* This routine splits the command into the upper and lower    
;* nibbles and sends them to the LCD, upper nibble first.      
;**************************************************************
scmd
Send_Cmd
	movwf   Char            ; Character to be sent is in W
	call    Wait_Busy       ; Wait for LCD to be ready
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	movf    Char,w          
	andlw   0xF0            ; Get upper nibble
	iorwf   LCD_DATA,F      ; Send data to LCD
	bcf     RW    ; Set LCD to write
	bcf     RS     ; Set LCD to command mode
	bsf     E      ; toggle E for LCD
	bcf     E
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	swapf   Char,W
	andlw   0xF0            ; Get lower nibble
	iorwf   LCD_DATA,F      ; Send data to LCD
	bsf     E      ; toggle E for LCD
	bcf     E
	return

;**************************************************************
;* This routine checks the busy flag, returns when not busy    
;*  Affects:                                                   
;*      Temp - Returned with busy/address                      
;**************************************************************

Wait_Busy
	bsf     STATUS, RP0     ; Select Register page 1
	movlw   0xf0            ; Set port to input
	iorwf   LCD_DATA_TRIS,W ; Only set upper half of port
	movwf   LCD_DATA_TRIS
	bcf     STATUS, RP0     ; Select Register page 0
	bcf     RS    		; Set LCD for Command mode
	bsf     RW   		; Setup to read busy flag
	bsf     E     		; Set E high
	movf    LCD_DATA, W     ; Read upper nibble busy flag, DDRam address
	bcf     E     		; Set E low
	andlw   0xF0            ; Mask out lower nibble
	movwf   Temp
	bsf     E     		; Toggle E to get lower nibble
	swapf   LCD_DATA, w     ; Read lower nibble busy flag, DDRam address
	bcf     E
	andlw   0x0F            ; Mask out upper nibble
	iorwf   Temp,f          ; Combine nibbles, RES in Temp !!!!!!!!!!!!!!!
	btfsc   Temp, 7         ; Check busy flag, high = busy
	goto    Wait_Busy       ; If busy, check again
	bcf     RW 
	bsf     STATUS, RP0     ; Select Register page 1
	movlw   0x0F
	andwf   LCD_DATA_TRIS,W
	movwf   LCD_DATA_TRIS   ; Set Port for output
	bcf     STATUS, RP0     ; Select Register page 0
	return

;**************************************************************
del10mks
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	goto $+1
	return
;**************************************************************
; Initilize the LCD Display Module
;**************************************************************
LCD_Init:

	bcf     E     ; Clear all controll lines
	bcf     RS
	bcf     RW
	call    delay20		; Wait for 40ms for LCD to get powered up
	call    delay20
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	movlw   0x030           ; Command for 4-bit interface high nibble
	iorwf   LCD_DATA,F      ; Send data to LCD      
	bsf     STATUS, RP0     ; Select Register page 1
	movlw   0x0F
	andwf   LCD_DATA_TRIS,W
	movwf   LCD_DATA_TRIS   ; Set Port for output
	BCF     STATUS, RP0     ; Select Register page 0
	bsf     E     		; Clock the initalize command to LCD module
	bcf     E
	call    delay5		; Delay for at least 4.1ms before continuing
	bsf     E     		; Clock the initalize command to LCD module
	bcf     E
	call    d100mks        	; delay for at least 100usec before continuing
	bsf     E     		; Clock the initalize command to LCD module
	bcf     E
	call    Wait_Busy       ; From here on, the Busy Bit will be valid.
	movlw   0x0f
	andwf   LCD_DATA,F      ; Clear the upper nibble
	movlw   0x020           ; Command for 4-bit interface high nibble 
	iorwf   LCD_DATA,F      ; Send data to LCD      
	bsf     E     		; Clock the initalize command to LCD module
	bcf     E
	movlw   FUNCTION_SET    ; Send the function set command 4-bit I/F, Font, Number of lines
	call    Send_Cmd        ; Can now use the Send_Cmd routine since Busy Bit Valid and in 4bit mode.
	
;       

	movlw	40	;    CGRAM
	call	Send_Cmd
	clrw
	call	sch_b
	movlw	7
	movwf	temp1
	movlw	1B	;     (0)
	call	sch_b	;   DDRAM 00
	decfsz	temp1
	goto	$-3
	clrw
	call	sch_b
	movlw	7
	movwf	temp1
	movlw	18	;     (1)
	call	sch_b	;   DDRAM 01
	decfsz	temp1
	goto	$-3


	movlw	80	;  DDRAM SET
	call	Send_Cmd
	return

